import click
import dockerapi
import warnings

import utils as u


@click.command()
@click.argument('image')
@click.option('--output', help='output image')
@click.option('--no-assert-hostname', is_flag=True,
              help='Disable hostname validation')
@click.option('--mount', multiple=True,
              help='Mount volume (format: "src:target[:rw,:ro]")')
def main(image, output, no_assert_hostname, mount):
    click.secho('\nUpdating image {} (to: {})'.format(
        image, output if output else 'same image'), fg='green')

    if not output:
        output = image

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")

        cli = dockerapi.get_cli(assert_hostname=not no_assert_hostname)

        # cache old options
        options = u.get_old_options(cli, image)

        # read code from stdin
        code = u.get_code()

        # execute code inside a container
        click.secho('Running code..., output:', fg='green')
        container_id = u.run_code_in_container(
            cli, image, code, mount, entrypoint=options['entrypoint'])

        # get logs
        exitcode = cli.wait(container=container_id)

        logs = cli.logs(container_id, stdout=True, stderr=True)

        if logs:
            if exitcode < 1:
                click.echo(logs)
            else:
                click.secho(logs, fg='red')
        else:
            click.secho('Nothing', bold=True)

        # commit running container to image
        if exitcode < 1:
            cli.commit(container=container_id, repository=output)
            cli.stop(container=container_id)

        cli.remove_container(container=container_id)

        # restore container's entrypoint and cmd
        if exitcode < 1:
            u.restore_image_options(cli, output, options)



from docker.client import Client
from docker.utils import kwargs_from_env


def get_cli(assert_hostname=True):
    kwargs = kwargs_from_env()

    if not assert_hostname:
        kwargs['tls'].assert_hostname = False

    cli = Client(**kwargs)

    return cli


def inspect_config(cli, image, param):
    return cli.inspect_image(image)['Config'].get(param)



import io
import json
import click
import dockerapi
import os.path

try:
    from shlex import quote
except ImportError:
    from pipes import quote


def get_old_options(cli, image):
    """ Returns Dockerfile values for CMD and Entrypoint
    """
    return {
        'cmd': dockerapi.inspect_config(cli, image, 'Cmd'),
        'entrypoint': dockerapi.inspect_config(cli, image, 'Entrypoint'),
    }


def get_code():
    """ Reads code from STDIN
    """
    code = ''
    with click.open_file('-', 'r') as f:
        code += f.read()
    return code


def restore_image_options(cli, image, options):
    """ Restores CMD and ENTRYPOINT values of the image

    This is needed because we force the overwrite of ENTRYPOINT and CMD in the
    `run_code_in_container` function, to be able to run the code in the
    container, through /bin/bash.
    """
    dockerfile = io.StringIO()

    dockerfile.write(u'FROM {image}\nCMD {cmd}'.format(
        image=image, cmd=json.dumps(options['cmd'])))

    if options['entrypoint']:
        dockerfile.write(
            '\nENTRYPOINT {}'.format(json.dumps(options['entrypoint'])))

    cli.build(tag=image, fileobj=dockerfile)


def run_code_in_container(cli, image, code, mount, entrypoint):
    """ Run `code` in a container, returning its ID
    """
    kwargs = {
        'image': image,
    }

    if entrypoint:
        kwargs['entrypoint'] = '/bin/bash'
        kwargs['command'] = '-c {}'.format(quote(code))
    else:
        kwargs['command'] = '/bin/bash -c {}'.format(quote(code))

    if mount:
        binds = []
        volumes = []

        for m in mount:
            part = m.split(':')

            if len(part) == 3:
                pass
            elif len(part) == 2:
                part.append('rw')
            else:
                raise

            src, target, mode = part
            src = os.path.abspath(os.path.expanduser(src))

            binds.append('{}:{}:{}'.format(src, target, mode))
            volumes.append(target)

        kwargs['host_config'] = cli.create_host_config(binds=binds)
        kwargs['volumes'] = volumes

    container = cli.create_container(**kwargs)
    container_id = container['Id']

    cli.start(container=container_id)

    return container_id



from setuptools import setup, find_packages


VERSION = '0.1.1'


setup(
    name='docker-run-build',
    version=VERSION,
    description="Incrementally build Docker images",
    author="Caio Ariede",
    author_email="caio.ariede@gmail.com",
    url="http://github.com/caioariede/docker-run-build",
    license="MIT",
    zip_safe=False,
    platforms=["any"],
    packages=find_packages(),
    entry_points={
        'console_scripts': ['docker-run-build=docker_rb.cli:main'],
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
    ],
    include_package_data=True,
    install_requires=[
        'six',
        'docker-py==1.6.0',
        'click==6.2',
    ],
)
